#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

//for transformation
#include <libxslt/transform.h>
#include <libxslt/xsltutils.h>
static GtkTextBuffer *buffer;
static GtkWidget *dialog_open;
static GtkWidget *dialog_save;

static void
printlog (const gchar * text)
{
  printf ("%s", text);
  GtkTextIter end;
  gtk_text_buffer_get_end_iter (buffer, &end);
  gtk_text_buffer_insert (buffer, &end, text, -1);
}

extern int xmlLoadExtDtdDefaultValue;

static int
byojtransform (const gchar * xmlFileName, const gchar * xsltFileName,
	       gchar * OutputFileName)
{
  printlog ("...Setup Parameters\n");
  const char *params[16 + 1];
  int nbparams = 0;
  xsltStylesheetPtr cur = NULL;
  xmlDocPtr doc, res;

  params[nbparams] = NULL;
  xmlSubstituteEntitiesDefault (1);
  xmlLoadExtDtdDefaultValue = 1;

  printlog ("...Loading Stylesheet\n");
  cur = xsltParseStylesheetFile ((const xmlChar *) xsltFileName);

  printlog ("...Loading XML\n");
  doc = xmlParseFile (xmlFileName);

  printlog ("...Applying Stylesheet\n");
  res = xsltApplyStylesheet (cur, doc, params);
  xsltSaveResultToFile (stdout, res, cur);

  printlog ("...Saving Tranformation\n");
  xsltSaveResultToFilename (OutputFileName, res, cur, 0);

  printlog ("...Freeing Memory\n");
  xsltFreeStylesheet (cur);
  xmlFreeDoc (res);
  xmlFreeDoc (doc);

  xsltCleanupGlobals ();
  xmlCleanupParser ();

  printlog ("...Successful Transformation\n");
  return (0);
}

void
set_buffer (GtkWidget * widget)
{
  GtkWidget *textview_log =
    lookup_widget (GTK_WIDGET (widget), "textview_log");
  GtkTextTagTable *tagtable = gtk_text_tag_table_new ();
  buffer = gtk_text_buffer_new (tagtable);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW (textview_log), buffer);
  gtk_text_buffer_set_text (buffer, "Buffer initialized\n", -1);

  dialog_open = create_filechooserdialog_open ();
  dialog_save = create_filechooserdialog_save ();
}

void
on_btn_xml_clicked (GtkButton * button, gpointer user_data)
{
  if (gtk_dialog_run (GTK_DIALOG (dialog_open)) == GTK_RESPONSE_OK)
    {
      gtk_widget_hide (dialog_open);

      gchar *filename;
      filename =
	gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog_open));

      GtkWidget *entry = lookup_widget (GTK_WIDGET (button), "entry_xml");
      gtk_entry_set_text (GTK_ENTRY (entry), filename);

      g_free (filename);
    }
  else
    {
      gtk_widget_hide (dialog_open);
    }
}


void
on_btn_xsl_clicked (GtkButton * button, gpointer user_data)
{
  if (gtk_dialog_run (GTK_DIALOG (dialog_open)) == GTK_RESPONSE_OK)
    {
      gtk_widget_hide (dialog_open);

      char *filename;
      filename =
	gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog_open));

      GtkWidget *entry = lookup_widget (GTK_WIDGET (button), "entry_xsl");
      gtk_entry_set_text (GTK_ENTRY (entry), filename);

      g_free (filename);
    }
  else
    {
      gtk_widget_hide (dialog_open);
    }
}


void
on_btn_transform_clicked (GtkButton * button, gpointer user_data)
{
  printlog ("\nStarted Transformation\n");
  if (gtk_dialog_run (GTK_DIALOG (dialog_save)) == GTK_RESPONSE_OK)
    {
      gtk_widget_hide (dialog_save);

      char *filename_output;
      filename_output =
	gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog_save));

      GtkWidget *entry_xml = lookup_widget (GTK_WIDGET (button), "entry_xml");
      const gchar *filename_xml = gtk_entry_get_text (GTK_ENTRY (entry_xml));
      if (g_file_test (filename_xml, G_FILE_TEST_IS_REGULAR))
	{
	  printlog (".XML File Exists\n");
	}
	else
	{
        printlog("Transformation Canceled: XML File does NOT exist");
        return;
    }

      GtkWidget *entry_xsl = lookup_widget (GTK_WIDGET (button), "entry_xsl");
      const gchar *filename_xsl = gtk_entry_get_text (GTK_ENTRY (entry_xsl));
      if (g_file_test (filename_xml, G_FILE_TEST_IS_REGULAR))
	{
	  printlog (".XSL File Exists\n");
	}
		else
	{
        printlog("Transformation Canceled: XSL File does NOT exist");
        return;
    }


      byojtransform (filename_xml, filename_xsl, filename_output);

      g_free (filename_output);

      printlog ("Transformation Completed\n");
    }
  else
    {
      gtk_widget_hide (dialog_save);
      printlog ("Transformation Canceled: File not selected\n");
    }
}


gboolean
on_window_xslt_destroy_event (GtkWidget * widget,
			      GdkEvent * event, gpointer user_data)
{
  gtk_main_quit ();
  return FALSE;
}


void
on_btn_about_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *dialog = create_aboutdialog1 ();
  gtk_dialog_run (GTK_DIALOG (dialog));
}


void
on_btn_close_clicked (GtkButton * button, gpointer user_data)
{
  gtk_main_quit ();
}
